# Simple metadata-extractor extension
This extension automatically extracting the metadata of a file. Which can be used for creating an asset or for calculating the FAIRness score.
We test the metadata-extractor extension using existing EDC sample [transfer-01-file-transfer](sample/transfer/transfer-01-file-transfer). 
The sample transmits a test file from one connector to another connector. Before transferring the file it adds the file as an asset but while creating the asset object we have to hardcode the metadata of the file.
By using the extension we automate the asset creation process. Another usecase of extracted metatdata is for calculating the fairness score. The [FairService](metadata-extractor/src/main/java/org/fraunhofer/fit/extension/extractor/FairService.java) class is based on  [Fairness evaluator](https://gitlab.fit.fraunhofer.de/abu.ibne.bayazid/fairevaluator) developed by Abu Ibne Bayazid. 
We use this repository with minor changes to calculate the Fairness score in our extension.

In our extension, we focused on both retrievable and extractable metadata. For now, extractable metadata only support PDF and Word documents; these formats are very commonly used types of files; especially for research in articles or reports. More detail about extension can be found in this [document](https://docs.google.com/document/d/1FRK8om-RnSXhTE8Qzupt6az1xSpNtqhMPvlIQTz3bGY/edit#)

## How to use the extension
To extract metadata form text file we used NLP/ML and in order to execute NLP/ML models for the metadata extraction python was used for the task due to its inherent flexibility and an extensive collection of NLP/ML packages;

Before we use the extension, a recent Python version needs to be [downloaded](https://www.python.org/downloads/) and installed in the system; as well as the required packages which are contained in the [requirements.txt](metadata-extractor/src/main/resources/requirement.txt) file,
the whole list of requirements could be installed using the following command ```pip install -r requirements.txt```.

The [text_metadata_extractor.py](metadata-extractor/src/main/resources/text_metadata_extractor.py) python file contains a set of functions that extract metadata (title, keywords, summary…) from the pdf and Word document.
This file is invoked by the main Java class. There are several ways of calling Python scripts in a Java class; the one that we chose consists of using a class called `ProcessBuilder`
then we afterward redirect the output of the Python script to the Java output to be used for the next step.

### Approach 1:  Use extension as an Api.
Extension can be used directly as an Api by passing the asset path, with other optional parameter. Turn off the provider and consumer connector and build and run metadata-extractor extension.

```bash
.\gradlew metadata-extractor:build
java -Dedc.fs.config=metadata-extractor/config.properties -jar metadata-extractor/build/libs/file-metadata-extractor.jar
```
In another terminal we can call the Api by passing the assetPath
```bash
curl -X GET "http://localhost:5000/api/extract-metadata?assetPath=C:\\Users\\{user}\\Desktop\\english-test.pdf&assetPid=https://doi.org/10.48550/arXiv.2302.12813&assetDataLink=https://arxiv.org/pdf/2302.12813.pdf"
```
This api will return the asset properties and fairness score. Below we can see an example To test the extension using English document we used [this](https://arxiv.org/pdf/2302.12813.pdf) paper and got the following properties:
![image 2](https://github.com/eclipse-edc/Connector/assets/13449847/16687a4a-8b71-4ed7-8aab-845262eb3866)

### Approach 2:  Use extension as a dependency.
The extensions can be used by adding the dependency to the Gradle file of other extensions.
In this case, the [transfer-file-local](sample/transfer/transfer-01-file-transfer/transfer-file-local) extension (it's part of the provider connector) will use the metadata-extractor.
We add the dependency by adding the following line into the [Gradle file](sample/transfer/transfer-01-file-transfer/transfer-file-local/build.gradle.kts).

```kotlin
implementation(project(":metadata-extractor"))
```

And then inject a dependency  of ```class MetadataExtractor``` by adding the following line in [FileTransferExtension](sample/transfer/transfer-01-file-transfer/transfer-file-local/src/main/java/org/eclipse/edc/sample/extension/api/FileTransferExtension.java)

```java
@Inject
private MetadataExtractor metadataExtractor;
```
Once we inject the dependency we have the object of ```class MetadataExtractor```. Using this object we can add properties automatically while creating an asset.
```java
        var metadata = metadataExtractor.getMetadata(assetPath.toString(), assetpid, assetDataLink, assetVersion);
        var asset = Asset.Builder.newInstance()
        .id(assetId)
        .contentType(metadata.remove("mimeType").toString())
        .name(metadata.remove("name").toString())
        .description(metadata.remove("description").toString())
        .version(metadata.remove("version").toString())
        .properties(metadata)
        .build();
```
#### Running the connectors:

First, we have to build the jar file for provider and consumer connector:
```bash
.\gradlew sample:transfer:transfer-01-file-transfer:file-transfer-provider:build
.\gradlew sample:transfer:transfer-01-file-transfer:file-transfer-consumer:build
````

Now we run the provider connector in one terminal.
```bash
java -Dedc.fs.config=sample/transfer/transfer-01-file-transfer/file-transfer-provider/config.properties -jar sample/transfer/transfer-01-file-transfer/file-transfer-provider/build/libs/provider.jar
```
And consumer connector in another terminal.
```bash
java -Dedc.fs.config=sample/transfer/transfer-01-file-transfer/file-transfer-consumer/config.properties -jar sample/transfer/transfer-01-file-transfer/file-transfer-consumer/build/libs/consumer.jar
```
Provider connector creates the in-memory asset object with the help of [metadtata-extractor](metadata-extractor) and [file-local-transfer](sample/transfer/transfer-01-file-transfer/transfer-file-local) extension.

Now in third terminal we should be able to get providers catalog detail on consumer side.
```bash
curl -X POST "http://localhost:9192/management/v2/catalog/request" --header "X-Api-Key: password" --header "Content-Type: application/json" --data-raw "{\"@context\": {\"edc\": \"https://w3id.org/edc/v0.0.1/ns/\"}, \"providerUrl\": \"http://localhost:8282/protocol\",\"protocol\": \"dataspace-protocol-http\"}"
```
This api will return the asset, contracts, policies detail available in the provider catalog. Below we can see an example To test the extension using English document we used [this](https://arxiv.org/pdf/2302.12813.pdf) paper and got the following properties:
![image 1](https://github.com/eclipse-edc/Connector/assets/13449847/8123b2af-1f98-48b8-826c-054825b4b856)


#### Note:
The Sample support filesystem-based-configuration and source file path should be updated [here](sample/transfer/transfer-01-file-transfer/file-transfer-provider/config.properties) and the target file path should be [here](sample/transfer/transfer-01-file-transfer/filetransfer.json).

Now that the asset has been created we can run the consumer connector and request to transfer the file. 
This process is the same as transfer-01-file-transfer and is out of the scope of this documentation. Only difference it `transfer` module is inside `sample/transfer`, so these command will change:
```bash
#contract contractnegotiations
curl -X POST -H "Content-Type: application/json" -H "X-Api-Key: password" -d @sample/transfer/transfer-01-file-transfer/contractoffer.json "http://localhost:9192/management/v2/contractnegotiations"
#transferprocesses
curl -X POST -H "Content-Type: application/json" -H "X-Api-Key: password" -d @sample/transfer/transfer-01-file-transfer/filetransfer.json "http://localhost:9192/management/v2/transferprocesses"
```
Please find the steps to run the complete file transfer process [here](sample/transfer/transfer-01-file-transfer/README.md).